import fs from "node:fs";

export type RotorValues = {
	name: string;
	values: string[];
	turnovers: string[];
};

export type ReflectorValues = {
	name: string;
	reflector: Map<string, string>;
};

function loadFile(filename: string) {
	const data = fs.readFileSync(filename, "utf-8");
	let row: string[] = [];
	const table: string[][] = [];
	let cell = "";

	for (const char of data) {
		if (char !== "," && char !== "\n") {
			cell += char;
		} else {
			row.push(cell);
			cell = "";

			if (char === "\n") {
				table.push(row);
				row = [];
			}
		}
	}

	// if the file ends in a line break, this isn't needed but if it doesn't, then there
	// is a rotor hanging out at the end that still needs to be added.
	if (row.length > 0) {
		row.push(cell);
		table.push(row);
	}

	return table;
}

export function buildRotors(filename: string) {
	const rotors: RotorValues[] = [];
	const table = loadFile(filename);

	for (const [name, valueString, turnovers] of table) {
		rotors.push({
			name,
			values: valueString.split(""),
			turnovers: turnovers.split(""),
		});
	}

	return rotors;
}

export function buildReflectors(filename: string) {
	const reflectors: ReflectorValues[] = [];
	const table = loadFile(filename);

	for (const [name, valueString] of table) {
		const values = valueString.split("");
		const reflector = { name, reflector: new Map() };

		for (let i = 0; i < values.length; i++) {
			const a = String.fromCharCode("A".charCodeAt(0) + i);
			const b = values[i];

			reflector.reflector.set(a, b);
			reflector.reflector.set(b, a);
		}

		reflectors.push(reflector);
	}

	return reflectors;
}
